/*=======================================================
H8/3048F AD Convert to Temperature prog. 
                         (c)2009.4.1. MI-TAKE/tminobe
                        
  target:AKIDUKI 3048F + Mother kit, with C compiler

  LCD pin.				LED
 	D0 :P3-0			LED1 :P5-0 = ADC timing marker
 	D1 :P3-1			
 	D2 :P3-2			
  	D3 :P3-3			
  	RS :P3-4			
  	E  :P3-5			
  	R/W:GND				
  	Vcc:Vdd 			
========================================================*/
#include "STDIO.H"
#include "3048F.H"
#define VresionNo "V1.2R7"
#define LCD_RS 0x10				/* P3-4 */
#define LCD_E  0x20				/* P3-5 */
#define LCDMASK 0xC0			/* P3-6,7 */
#define  LED1  P5.DR.BIT.B0		/* RxD LED */
#define  LED2  P5.DR.BIT.B1		/* TxD LED */
#define addrah (volatile unsigned char *)0xffffe0
#define addral (volatile unsigned char *)0xffffe1

int dumy1=0;					/* linker worning ΍ */
int dumy2  ;					/* linker worning ΍ */

void PortInit(void);
void InitLCD();
void PutLCD(char c);
void LocateLCD(int x,int y);
void wait_m(int c);
void initSCI(int b);

void initIO(void)
{
	P5.DDR = 0xff;					/* P5 output mode */
	P5.DR.BYTE = 0x00;				/* LED1,2 off */
	P3.DDR = 0xff;					/* P3 output mode */
	P3.DR.BYTE = 0;					/* LCD port */
	InitLCD();						/* LCD init. */
	initSCI(51);					/*init SCI(9600PBS) */
}

/*=======================================================*/
/* LCD data output : 8bit  */
/*=======================================================*/

void LCDOut8(int rs,int ch_data)
{
	int w_mode;
	unsigned char p3d;

	p3d = P3.DR.BYTE; 			/* push P3data */
	if(rs) w_mode = (p3d & LCDMASK) | LCD_RS;
	else w_mode = (p3d & LCDMASK);
	P3.DR.BYTE = ch_data | w_mode | LCD_E;
	P3.DR.BYTE = ch_data | w_mode;
	P3.DR.BYTE = p3d;			 /* pop P3data */
	wait_m(1);
}
	
/*=======================================================*/
/* LCD data output : 4bit  */
/*=======================================================*/
void LCDOut4(int rs,int ch_data)
{
	int w_mode;
	unsigned char p3d;
	char low,hig;

	p3d = P3.DR.BYTE; 			/* push P3data */

	if(rs) w_mode = (p3d & LCDMASK) | LCD_RS;
	else w_mode = (p3d & LCDMASK);

	hig = ((ch_data>>4)&0x0f) | w_mode;
	low = (ch_data&0x0f) | w_mode;
	P3.DR.BYTE = hig | LCD_E;
	P3.DR.BYTE = hig;
	P3.DR.BYTE = low | LCD_E;
	P3.DR.BYTE = low;
	P3.DR.BYTE = p3d; 			/* push P3data */
	wait_m(1);
}

/*=======================================================*/
/* LCD intialis */
/*=======================================================*/
void InitLCD()
{
	int i;
	wait_m(10);
	for(i=0;i<3;i++){		/* 4bit mode */
		LCDOut8(0,0x03); wait_m(1);
	}
	LCDOut8(0,0x02);		/* LCD init last */
	LCDOut4(0,0x28);		/* 4Bit mode,2Line disp,5*8Char */
	LCDOut4(0,0x0c);		/* Disp on, Cursor off */
	LCDOut4(0,0x06);		/* Entry mode set */
	LCDOut4(0,0x01);		/* cls */
	LCDOut4(0,0x02); 		/* cursor home position */
}

/*=======================================================*/
/* LCD clear */
/*=======================================================*/
void ClearLCD()
{
	LCDOut4(0,0x01);
	LCDOut4(0,0x02);
}

/*=======================================================*/
/* LCD char data output */
/*=======================================================*/
void PutLCD(char c)
{
	if( c == '\f' ) ClearLCD();
	else if( c == '\n' ) ClearLCD();
	else if( c == '\r' ) ClearLCD();
	else LCDOut4(1,c);
}

/*=======================================================*/
/* cursor pos set */
/*=======================================================*/
void LocateLCD(int x,int y)
{
	LCDOut4(0,0x80 + y*0x40 + x);
}
	
/*=======================================================*/
/* LCD string data output */
/*=======================================================*/
void PrintLCD(char *buf )
{
	int i;

	for(i=0;;i++){
		if( buf[i] == 0 ) break;
		else if( buf[i] == '\n' ) LocateLCD(0,1);
		else if( buf[i] == '\r' ) LCDOut4(0,0x2);
		else if( buf[i] == '\f' ) ClearLCD();
		else if( buf[i] == '\f' ) ClearLCD();
		else LCDOut4(1,buf[i]);	
	}
}

/*=======================================================*/
void  wait_m(int c)
{
	int i,j;
	for(j=0;j<c;j++)
		for(i=0;i<1500;i++);
}

/*=======================================================*/
/*  t second timer */
/*=======================================================*/
void WaitSecond(int t)
{
	int i;
	for(i=0; i<t; i++) wait_m(1000);
}

/*=======================================================*/
/* binary to ascii */
/*=======================================================*/
void BinToAscii(char *buf,long d)
{
	int i,j=0;
	for(i=7;i>=0;i--){
		if(i==2) buf[i]='.';		/* _}	 */
		else{
			buf[i] = (d%10L) +48;
			d = d/10L;
		}
	}
	buf[5]=' ';
	buf[6]=0;
	buf[7]=0;
	buf[8]=0;
}

/*=======================================================*/
/* buf clear */
/*=======================================================*/
void BufClear(char *buf)
{
	int i;
	for(i=0;i<16;i++) buf[i]=0;
}

/*=======================================================*/
/* AD intialise */
/*=======================================================*/
void InitAD()
{
	AD.CSR.BIT.ADF  = 0;		/* AD end flug reset */
	AD.CSR.BIT.SCAN = 0;		/* single mode */
	AD.CSR.BIT.CKS  = 1;		/* convert time = 134State */
	AD.CSR.BIT.CH   = 0;		/* input ch. = AN0 */
}

/*=======================================================*/
/* AD data */
/*=======================================================*/
long GetAD()
{
	long ad_h,ad_l;
	long ad_varue;
	LED1 = 1;
	AD.CSR.BIT.ADST = 1;			/* start AD */
	while (AD.CSR.BIT.ADF == 0); 	/* AD convert end ? */
	AD.CSR.BIT.ADF = 0x00; 			/* AD end flug reset */
	ad_h=(unsigned)*addrah;
	ad_l=(unsigned)*addral;
	ad_varue= (ad_h*256 + ad_l) >>6;	/* right justfy 6bit */
	wait_m(10); LED1 = 0;
	return(ad_varue); 
}

/*=======================================================*/
/* initialize SCI */
/*=======================================================*/
void initSCI(int b)
{
    int i;
	SCI1.SCR.BYTE = SCI1.SMR.BYTE = 0;	/* init. SCI(1) */
    SCI1.BRR = b;						/* bit rate set */
	wait_m(10);
    SCI1.SCR.BYTE = 0x30;				/* enable TxD */
    SCI1.SSR.BYTE;						/* dummy read */
    SCI1.SSR.BYTE = 0x80; 				/* error flug clear */
}

/*=======================================================*/
/* send char TxD */
/*=======================================================*/
void PutcSCI(char c) 
{
    while (SCI1.SSR.BIT.TDRE == 0) ;	/* 0:already exists. wait 1 */
    SCI1.TDR = c;						/* send TxD */
    SCI1.SSR.BIT.TDRE = 0;				/* TDRE bit clear */
}

/*=======================================================*/
/* send string TxD */
/*=======================================================*/
void PrintSCI(char *st)
{
    while (*st) PutcSCI(*st++);
}

/*=======================================================*/
/* check RxD  */
/*=======================================================*/
char GetSCI(void)
{
    char c;
	int cnt;
	
	for(cnt=0; cnt<100 ;cnt++) {
		if((SCI1.SSR.BYTE & 0x78) != 0) cnt=101; /* Rx flug check */
	}
    if (SCI1.SSR.BIT.RDRF == 1) {     /* data rady Rx */
        c = SCI1.RDR;                 /* RxD =>c */
        SCI1.SSR.BIT.RDRF = 0;        /* Rx flug clear */
    } 
	else if(cnt<100) {
        SCI1.SSR.BYTE &= 0x80;        /* error flug clear */
        c = 0;
    }
	return(c);
}


/*=======================================================*/
/*=======================================================*/
void main(void)
{
	long a;
	char buff[16]={0},c;
	int cnt=0;

	initIO();  InitAD();
	LocateLCD(0,0); PrintLCD("H8/Thermo "); PrintLCD(VresionNo);

	while(1){ 
		a = GetAD() * 9611;			/* 4.95V,Gain5.0 1024i10bit)ŊZ  */
		BinToAscii(buff,a);
		if(cnt>50){
			LocateLCD(0,1);
			PrintLCD( "Temp= " ); PrintLCD(buff); LCDOut4(1,0xdf); LCDOut4(1,'C');
			cnt=0;
		}
        c = GetSCI();					/* RxD */
    	if(c=='T') {
    		LED1=1;
    		PutcSCI('T'); PrintSCI(buff); PutcSCI('\n'); PutcSCI('\r');
    		LED1=0;
    	}
    	else if(c=='S') {
    		LED1=1;
    		PutcSCI('S'); PrintSCI(buff); PutcSCI('\n'); PutcSCI('\r');
    		LED1=0;
    	}
	    SCI1.SSR.BYTE &= 0x80;        /* error flug clear */
		BufClear(buff);
		cnt++;
	}
}


